@@ -1,11 +1,14 @@
-use crate::directory_sync::{Group, User};
+use crate::directory_sync::{
+    Group, ListDirectoriesOptions, ListDirectoriesResponse, ListUsersOptions, ListUsersResponse,
+    User,
+};
 use crate::error::WorkOSError;
 use crate::organizations::Organization;
 use anyhow::{anyhow, Context, Result};
 use futures::stream::{self};
 use futures::{StreamExt, TryStreamExt};
 use std::collections::HashMap;
-use surf::StatusCode;
+use surf::{Response, StatusCode};
 
 const DEFAULT_BASE_URL: &str = "https://api.workos.com";
 
@@ -25,6 +28,46 @@ impl Client {
 }
 
 impl Client {
+    pub async fn list_directories(
+        &self,
+        options: ListDirectoriesOptions,
+    ) -> Result<ListDirectoriesResponse, WorkOSError> {
+        let mut req = surf::get(format!("{}/directories", &self.base_url))
+            .header("Authorization", format!("Bearer {}", &self.api_key))
+            .build();
+
+        req.set_query(&options)
+            .map_err(|e| anyhow!(e))
+            .context("unable to add query parameters")
+            .map_err(WorkOSError::Unknown)?;
+
+        let mut res = surf::client()
+            .send(req)
+            .await
+            .map_err(|e| anyhow!(e))
+            .context("unable to perform request")
+            .map_err(WorkOSError::Unknown)?;
+
+        examine_response(&mut res, None).await?;
+
+        res.body_json()
+            .await
+            .map_err(|e| anyhow!(e))
+            .context("unable to parse response")
+            .map_err(WorkOSError::Unknown)
+    }
+
+    pub async fn delete_directory(&self, directory_id: &str) -> Result<(), WorkOSError> {
+        let mut res = surf::delete(format!("{}/directories/{}", &self.base_url, directory_id))
+            .header("Authorization", format!("Bearer {}", &self.api_key))
+            .await
+            .map_err(|e| anyhow!(e))
+            .context("unable to perform request")
+            .map_err(WorkOSError::Unknown)?;
+
+        examine_response(&mut res, Some(directory_id.to_string())).await
+    }
+
     pub async fn get_user(&self, user_id: &str) -> Result<User, WorkOSError> {
         let mut res = surf::get(format!("{}/directory_users/{}", &self.base_url, user_id))
             .header("Authorization", format!("Bearer {}", &self.api_key))
@@ -33,14 +76,7 @@ impl Client {
             .context("unable to perform request")
             .map_err(WorkOSError::Unknown)?;
 
-        match res.status() {
-            StatusCode::Ok => Ok(()),
-            StatusCode::NotFound => Err(WorkOSError::NotFound(user_id.to_string())),
-            _ => Err(WorkOSError::Unknown(anyhow!(format!(
-                "received unexpected status '{}'",
-                res.status()
-            )))),
-        }?;
+        examine_response(&mut res, Some(user_id.to_string())).await?;
 
         res.body_json()
             .await
@@ -65,6 +101,35 @@ impl Client {
             .await
     }
 
+    pub async fn list_users(
+        &self,
+        options: ListUsersOptions,
+    ) -> Result<ListUsersResponse, WorkOSError> {
+        let mut req = surf::get(format!("{}/directory_users", &self.base_url))
+            .header("Authorization", format!("Bearer {}", &self.api_key))
+            .build();
+
+        req.set_query(&options)
+            .map_err(|e| anyhow!(e))
+            .context("unable to add query parameters")
+            .map_err(WorkOSError::Unknown)?;
+
+        let mut res = surf::client()
+            .send(req)
+            .await
+            .map_err(|e| anyhow!(e))
+            .context("unable to perform request")
+            .map_err(WorkOSError::Unknown)?;
+
+        examine_response(&mut res, None).await?;
+
+        res.body_json()
+            .await
+            .map_err(|e| anyhow!(e))
+            .context("unable to parse response")
+            .map_err(WorkOSError::Unknown)
+    }
+
     pub async fn get_group(&self, group_id: &str) -> Result<Group, WorkOSError> {
         let mut res = surf::get(format!("{}/directory_groups/{}", &self.base_url, group_id))
             .header("Authorization", format!("Bearer {}", &self.api_key))
@@ -73,14 +138,7 @@ impl Client {
             .context("unable to perform request")
             .map_err(WorkOSError::Unknown)?;
 
-        match res.status() {
-            StatusCode::Ok => Ok(()),
-            StatusCode::NotFound => Err(WorkOSError::NotFound(group_id.to_string())),
-            _ => Err(WorkOSError::Unknown(anyhow!(format!(
-                "received unexpected status '{}'",
-                res.status()
-            )))),
-        }?;
+        examine_response(&mut res, Some(group_id.to_string())).await?;
 
         res.body_json()
             .await
@@ -119,14 +177,7 @@ impl Client {
         .context("unable to perform request")
         .map_err(WorkOSError::Unknown)?;
 
-        match res.status() {
-            StatusCode::Ok => Ok(()),
-            StatusCode::NotFound => Err(WorkOSError::NotFound(organization_id.to_string())),
-            _ => Err(WorkOSError::Unknown(anyhow!(format!(
-                "received unexpected status '{}'",
-                res.status()
-            )))),
-        }?;
+        examine_response(&mut res, Some(organization_id.to_string())).await?;
 
         res.body_json()
             .await
@@ -135,3 +186,24 @@ impl Client {
             .map_err(WorkOSError::Unknown)
     }
 }
+
+async fn examine_response(response: &mut Response, id: Option<String>) -> Result<(), WorkOSError> {
+    let status = response.status();
+    match status {
+        StatusCode::Ok => Ok(()),
+        StatusCode::BadRequest => Err(WorkOSError::NotAcceptable),
+        StatusCode::Unauthorized => Err(WorkOSError::InvalidApiKey),
+        StatusCode::Forbidden => Err(WorkOSError::IncorrectPermissions),
+        StatusCode::NotFound => Err(WorkOSError::NotFound(
+            id.unwrap_or_else(|| "unknown".to_string()),
+        )),
+        _ => Err(WorkOSError::Unknown(anyhow!(format!(
+            "received unexpected status '{}' : {}",
+            status,
+            response
+                .body_string()
+                .await
+                .unwrap_or_else(|_| "unable to read response".to_string())
+        )))),
+    }
+}
